import unittest
import openpyxl.styles
import openpyxl.utils

from hu2.excel2 import ExcelTable, ExcelStyle, COLOR_GREEN, COLOR_DARKGREEN, COLOR_DARKBLUE

headFont = openpyxl.styles.Font(color=COLOR_GREEN,bold=True,size=16)
headAlignment = openpyxl.styles.Alignment(horizontal='center', vertical='center', wrap_text=True)

headFill2 = openpyxl.styles.GradientFill(stop=('FFFFFF','99ccff','000000'))  #渐变颜色
headFill  = openpyxl.styles.PatternFill('solid',fgColor=COLOR_DARKBLUE,bgColor=COLOR_DARKGREEN) # solid时只有前景色有效
#                                                          前景色                  背景色

class TestExcel(unittest.TestCase):
    def test_excel(self):
        et = ExcelTable('excel.xlsx')
        et.set_column_width({'A': 10, 'B': 20, 'C': 30})
        et.set_head(['Aa', 'Bb', 'Cc'])
        et.set_body([['a', 'b', 'c'], (1, 2, 3)])
        et.set_head_style(ExcelStyle(font=headFont, alignment=headAlignment,fill=headFill,border=True))
        et.set_body_style(ExcelStyle(alignment=headAlignment,border=True))
        et.sheet.row_dimensions[3].height = 40 # 3-第三行，首行为第1行
        et.sheet.title='工作表11' # 设置工作表的名称，默认为 Sheet
        et.save()

    def test_utils(self):
        print(' index=1 -> ',openpyxl.utils.get_column_letter(1))
        print('letter=B -> ',openpyxl.utils.column_index_from_string('B'))

    def test_load_excel(self):
        wb = openpyxl.load_workbook("excel.xlsx")
        sheet = wb.active
        for col in sheet.iter_cols(): # _cells_by_col
            print(type(col),col,type(col[0]), col[0].value)
        for col in sheet.columns: # iter_cols
            print(type(col), col, type(col[0]), col[0].value)

        for row in sheet.iter_rows(): # _cells_by_row
            print(type(row), row, type(row[0]), row[0].value)
        for row1 in sheet.rows: # iter_rows
            print(type(row1), row1, type(row1[0]), row1[0])

        for row in sheet.values: # iter_rows(values_only=True) 返回不是 cell对象 而是 cell.value
            print(type(row), row)

        for t in sheet.tables:  # 插入表格后，可以查看到 不常用
            print(type(t),t)

if __name__ == '__main__':
    unittest.main()


from dataclasses import dataclass
from typing import Dict,Any

from openpyxl.styles import Alignment, Fill, Font, Border, Side
from openpyxl.workbook import Workbook
from openpyxl.cell import Cell



COLOR_BLACK = '00000000'  #COLOR_INDEX[0]
'''黑色'''

COLOR_WHITE = '00FFFFFF'  #COLOR_INDEX[1]
'''白色'''

COLOR_RED   = '00FF0000'  #COLOR_INDEX[2]
'''红色'''

COLOR_DARKRED = '00000000'  #COLOR_INDEX[8]
'''深红'''

COLOR_BLUE = '000000FF'  #COLOR_INDEX[4]
'''蓝色'''

COLOR_DARKBLUE = '000000FF'  #COLOR_INDEX[12]
'''深蓝'''

COLOR_GREEN = '0000FF00'  #COLOR_INDEX[3]
'''绿色'''
COLOR_DARKGREEN = '00FFFFFF'  #COLOR_INDEX[9]
'''深绿'''
COLOR_YELLOW = '00FFFF00'  #COLOR_INDEX[5]
'''黄色'''
COLOR_DARKYELLOW = '00808000'  #COLOR_INDEX[19]
'''深黄'''
sd = Side(style='thin')
BORDER = Border(left=sd, right=sd, top=sd, bottom=sd)

@dataclass
class ExcelStyle:
    '''支持 字体设置  填充设置  对其设置  有无边框设置'''
    font: Font = None
    fill: Fill = None
    alignment: Alignment = None
    border: bool = False
class ExcelTable:
    def __init__(self, file_name):
        self.head_list = None
        self.head_style = None
        self.body_grid = None
        self.body_style = None
        self.file_name = file_name
        self.wb = Workbook()
        self.sheet = self.wb.active
    def set_head_style(self,style:ExcelStyle):
        ''' 设置 列头区域单元格的 全局样式 '''
        self.head_style = style

    def set_head(self, head_list: [str]):
        ''' 设置列头区域单元格内容，数组 '''
        self.head_list = head_list

    def set_body(self, body_grid: [[Any]]):
        ''' 设置 数据区域的单元格内容，二维数组 '''
        self.body_grid = body_grid

    def set_body_style(self, style: ExcelStyle):
        ''' 设置 数据区域单元格的 全局样式 '''
        self.body_style = style


    def set_column_width(self, col_width_dict: Dict[str, int]):
        '''设置指定列的宽度，可以一次指定多列，dict中格式：{'A':30}'''
        for col_str, col_width in col_width_dict.items():
            self.sheet.column_dimensions[col_str].width = col_width

    @staticmethod
    def _set_style(style: ExcelStyle, cell:Cell):
        if style is None:
            return
        if style.fill is not None:
            cell.fill = style.fill
        if style.alignment is not None:
            cell.alignment = style.alignment
        if style.font is not None:
            cell.font = style.font
        if style.border:
            cell.border = BORDER
        return cell

    def save(self):
        '''
        将列头、数据区域 样式 和 内容 都设置到 Excel的sheet中，并保存到文件中
        :return:
        '''
        if self.head_list is not None:
            for index, head in enumerate(self.head_list):
                cell = self.sheet.cell(row=1, column=index + 1, value=str(head))
                ExcelTable._set_style(self.head_style, cell)

        if self.body_grid is not None:
            for row in self.body_grid:
                self.sheet.append(row)

        body_start_idx = 1 if self.head_list else 0
        for i,row in enumerate(self.sheet.rows):
            # 遍历每一行，对数据区域 继续遍历行中单元格 设置样式
            if i >= body_start_idx :
                for cell in row:
                    ExcelTable._set_style(self.body_style, cell)

        self.wb.save(self.file_name)



import sys
import datetime
import logging


# 运行辅助工具

def confirm(desc,input_tip=None):
    """
    提示并等待输入。特殊情况：当命令行运行增加--noconfirm时，用其等号后面的内容作为值，例如 --noconfirm=abc  即返回 abc
    :param desc: 功能介绍说明，会在确认执行时展示
    :param input_tip: 等待输入的提示文本
    :return:
    """
    print('运行参数：', sys.argv)
    for arg in sys.argv:
        if arg.startswith('--noconfirm'):
            ipt = arg.split('=')[1]
            return ipt

    print(desc)
    print("请输入并回车以继续:" if input_tip is None else input_tip, end='')
    txt = input()
    return txt


def log(*,logname_prefix=None,stream_level=logging.INFO,rootlogger_level=logging.DEBUG,logger_level_dict=dict(),log_single=True):
    """
    启用日志功能，控制台为 INFO 等级，输出到 err 流中
    :param logname_prefix: 日志文件名前缀，不设置时不启用文件日志。可以是单独的文件或含路径的文件 的名称前缀。生成的日志文件会在后面追加 [年月日时分秒.log]
    :param stream_level: 控制台日志等级，默认为INFO
    :param rootlogger_level: 根日志记录器等级，默认DEBUG
    :param logger_level_dict: 日志记录器等级字典，key-日志名称，value-日志等级，如logging.INFO
    :param log_single: 日志文件不增加时间戳，默认True，
    :return:
    """
    handlers = []
    if logname_prefix is not None:
        logfilename = logname_prefix + ( '1' if log_single else datetime.datetime.now().strftime("%Y%m%d%H%M%S") ) + '.log'
        fileHandler = logging.FileHandler(filename=logfilename, mode='w', encoding='utf-8')
        handlers.append(fileHandler)

    streamHandler = logging.StreamHandler(sys.stderr)
    streamHandler.setLevel(stream_level)
    handlers.append(streamHandler)
    for k,v in logger_level_dict.items():
        logging.getLogger(k).setLevel(v)

    logging.basicConfig(level=rootlogger_level, format='%(asctime)s - %(levelname)-8s - %(name)s - %(message)s',
                        handlers=handlers)


# 获取目录信息：(文件的总大小,文件个数，子文件夹个数)
#  不存在的目录，返回None
#  存在的目录，返回3个元素的元组
import os
import shutil


def dir_info(root_dir: str):
    """
    获取目录信息元组：(文件的总大小,文件个数，子文件夹个数)
    :param root_dir: 根目录
    :return: 元组 或 None
    """
    while True:
        # 检查参数 路径是否存在
        if not os.path.exists(root_dir):
            return 0,0,0
        # 检查参数 路径是否为文件
        if os.path.isfile(root_dir):
            return os.path.getsize(root_dir),1,0

        size = 0
        file_count = 0
        subdir_count = 0
        for cur_dir,sub_dirs,files in os.walk(root_dir):
            file_count += len(files)
            subdir_count += len(sub_dirs)
            for name in files:
                size += os.path.getsize(os.path.join(cur_dir, name))
        return size,file_count,subdir_count
    pass

def insert_path(path,inst_dir):
    absPath = os.path.abspath(path)
    pathTup = os.path.split(absPath)
    return os.path.join(pathTup[0],inst_dir,pathTup[1])

def move_insert_path(path,inst_dir):
    newdir = insert_path(path,inst_dir)
    if os.path.exists(path):
        shutil.move(path,newdir)
        return (True,newdir)
    else:
        return (False,newdir)
def join(*paths):
    """与os.path.join 类似，担忧以下不同：
        1，遇到/或'\\\\'开头的路径段，会忽略前面参数的路径前缀
        2，遇到\和/混合使用时， 原始版本，原样处理， 本方法：会统一转为平台相关的分隔符
    """
    new_paths = []
    for path in paths:
        path = path.removeprefix("/")
        path = path.removeprefix("\\")
        new_paths.append(os.path.normpath(path))
    # print(new_paths,paths)
    return os.path.join(*new_paths)
def join_slash(*paths):
    return join(*paths).replace("\\","/")


import configparser
import logging
import os
import sys

from sqlalchemy import URL,create_engine

logger = logging.getLogger(__name__)

class ConfigInfo:

    def __init__(self, cfgfilename):
        """

        :param cfgfilename:
        """
        self.filename = cfgfilename
        print('运行参数：', sys.argv)
        for arg in sys.argv:
            if arg.startswith('--config'):
                self.filename = arg.split('=')[1]
        configfile = os.path.abspath(self.filename)
        config = configparser.ConfigParser()
        if os.path.exists(configfile):
            config.read(configfile, encoding='utf-8')
        else:
            print(f"配置文件 {configfile} 未找到")
            raise FileNotFoundError(configfile)

        print(type(config))
        self.config = config


    def create_db_engine(self):
        __db_user = self.config['db']['user']
        __db_pass = self.config['db']['pass']
        __db_host = self.config['db']['host']
        __db_port = self.config['db']['port']
        __db_database_name = self.config['db']['database_name']
        dburl = URL.create('mysql+pymysql', username=__db_user, password=__db_pass, host=__db_host,
                           database=__db_database_name, port=__db_port, query={"charset": 'utf8'})
        logger.warning('%s %s', type(dburl), dburl)
        xmdb_engine = create_engine(dburl)
        return xmdb_engine


from sqlalchemy import text


class DbWrapper:


    def __init__(self,db_engine):
        self.db_engine = db_engine
        pass


    def query_list(self, sqlstr:str, param_dict=None)->list:
        """ 执行 select sql,可能返回多行记录，无结果时返回空列表，注意返回的记录行对象无法直接被序列化为json """
        if param_dict is None:
            param_dict = dict()
        with self.db_engine.connect() as conn:
            rt = conn.execute(text(sqlstr),param_dict).mappings().fetchall()
            if rt is not None:
                return rt

        return []

    def update(self, sqlstr:str, param_dict=None)->int:
        """执行一条 insert/update/delete sql，返回执行影响条数 """
        if param_dict is None:
            param_dict = dict()
        with self.db_engine.connect() as conn:
            rt = conn.execute(text(sqlstr),param_dict)
            conn.commit()
            if rt is not None:
                return rt.rowcount
            else:
                return -1

    def insert(self, sqlstr:str, param_dict=None)->int:
        if param_dict is None:
            param_dict = dict()
        with self.db_engine.connect() as conn:
            rt = conn.execute(text(sqlstr),param_dict)
            conn.commit()
            if rt is not None:
                return rt.lastrowid
            else:
                return -1


import os


def strftime_nopad(date,fmtstr):
    """
    统一使用 %-d 格式来去掉前导0的处理。在windows平台中需要替换成 %#d
    :param date: datetime对象
    :param fmtstr: 日期格式字符串 例如 %Y-%m-%d %H:%M:%S
    :return: 日期字符串
    """
    if os.name == 'nt':
        fmtstr = fmtstr.replace('%-','%#')
    return date.strftime(fmtstr)


import json
from datetime import datetime

class DateEncoder(json.JSONEncoder):
    def default(self, obj):
        if isinstance(obj, datetime):
            return obj.strftime('%Y-%m-%d %H:%M:%S')
        return json.JSONEncoder.default(self, obj)

def todumps(obj) -> str :
    """
    将参数对象转换为json字符串
    datetime类型转为：年-月-日 时:分:秒
    中文内容原样输入，不进行\\uxxxx的Unicode编码转换
    :param obj:
    :return:
    """
    return json.dumps(obj, cls=DateEncoder, ensure_ascii=False)


from setuptools import setup, find_packages

setup(name='hu2',
      version='0.0.3',
      author='huzhe666',
      author_email='huzhe666@live.com',
      description='a small tool of python',
      long_description='<p>no time write the document</p>',
      packages=find_packages()
      )
